
#    This is a component of Spectrum.tcl

if {!$Started} {
    
    variable SpecName "*"
    variable SpecRawData {}
    variable Compression 1
    variable SpecData ""
    variable SpecCont ""
    variable SpecLength ""
    variable DataLength ""
    variable SpecTitle ""
    variable SpecShow 0
    variable SpectrumList {}
    variable DirectoryList {}
    variable SpecDim 1
    variable SpecB 0
    variable SpecR 1024
    variable SpecArgs {}
    variable DataList {}
    variable SearchName 0
    variable XMin 0
    variable XMax 65535
    variable YMin 0
    variable YMax 0
    variable CanvasX 512
    variable CanvasY 512
    variable SpecBase 0
    variable SpecRange 65536
    variable SpecBase0 0
    variable SpecRange0 256
    variable SpecBase 0
    variable SpecSize 256
    variable SpecRange 256
    variable SpecLimit 0
    variable WindowSize 0
    variable Skip0 0
    variable DefCont 0
    variable Showing 0
}
    

   proc do_view {args} {
       variable XMin
       variable XMax
       variable YMin
       variable YMax
       variable SpecSize
       variable SpecLimit
       variable WindowSize 

       set f [lreplace [first $args] 0 0]
   
       switch $f {
           "expand"        {do_expand}
            zoomin -
           "zoom in"       {do_zoom In}
            zoomout -
           "zoom out"      {do_zoom Out}
            times2         {set YMax [expr $YMax*2]}
            divide2        {set YMax [expr $YMax/2]}
            left           {set XMin [expr $XMin - ($WindowSize/4)]; set XMax [expr $XMax - ($WindowSize/4)]}
            right          {set XMin [expr $XMin + ($WindowSize/4)]; set XMax [expr $XMax + ($WindowSize/4)]}
            up             {set YMin [expr $YMin - ($SpecLimit/4)]; set YMax [expr $YMax - ($SpecLimit/4)]}
            down           {set YMin [expr $YMin + ($SpecLimit/4)]; set YMax [expr $YMax + ($SpecLimit/4)]}
            reset          {do_reset}
           "skip channel 0"    {do_skip "true"}
           "unskip channel 0"  {do_skip "false"}
            default         {InsertLog "Unknown view function $f"}
       }
   }
   
   proc do_arrange {args} {

       set f [lreplace [first $args] 0 0]
   
       switch $f {
           "remove all spectra"   {}
           default                {InsertLog "Unknown arrange function $f"}
       }
   }
   
   proc do_analysis {args} {
       variable SpecName
 
       set f [lreplace [first $args] 0 0]
   
       switch $f {
           integrate            {do_integrate}
           "peak to total"      {do_peaktototal}
          "max channel counts"  {set r [do_maxcounts]; InsertLog "Maximum channel counts for $SpecName: Ch [first $r] Contents [second $r]"}
          "total counts"        {InsertLog "Total channel counts for $SpecName: [do_totalcounts]"}
           default              {InsertLog "Unknown analysis function $f"}
       }
   }
   
   proc do_tagsfits {args} {

       set f [lreplace [first $args] 0 0]
   
       switch $f {
          "clear tags"    {do_cleartags}
           default        {InsertLog "Unknown tagsfits function $f"}
       }
   }
   
   proc do_actions1 {args} {
       variable SpecDim
       variable showit
       variable MemSas
       variable DiscSas
       variable DirName
       
       set f [lreplace [first $args] 0 0]
   
       switch $f {
           details   {do_details 0}
           title     {do_title 0}
           create    {
               if {$SpecDim == 1} {do_create1D}
               if {$SpecDim == 2} {do_create2D}
               set showit 0
           }
           delete     {do_delete; set showit 0}
           zero       {do_zero; set showit 0}
           inquire    {do_inquiry [SpectrumName]}
           inquireall {
              if {$MemSas == 1}  {do_inquiry *}
              if {$DiscSas == 1} {do_inquiry [file join $DirName *]}
           }
           reset   {do_reset}
           refresh {}
           default {InsertLog "Unknown analysis function $f"}
       }
   }
   
    proc do_actions2 {args} {
    
       set f [second [first $args]]
       set vl [third [first $args]]

       switch $f {
           tag     {do_pushtag $vl}
           default {InsertLog "Unknown analysis function $args ($f)  ($vl)"}       
       }
    }
    
    proc do_integrate {} {
       global env tcl_platform
       variable ntags
       variable tagx
    
      if {$ntags < 2} {InsertLog "2 limits are required"; return}
    
      set execute "exec integrate -spectrum [SpectrumName] -limit $tagx($ntags) $tagx([expr $ntags-1])"
      set rc [catch {eval $execute} m]
      
      if {$rc == 0} {
          InsertLog "$m"
      } else {
          InsertLog "Integrate returns an error: $m"
      }
    }
    
    proc do_peaktototal {} {
       global env tcl_platform
       variable ntags
       variable tagx
    
      if {$ntags < 2} {InsertLog "2 limits are required"; return}
      
      set execute "exec peak-to-total -spectrum [SpectrumName] -limit $tagx($ntags) $tagx([expr $ntags-1])"
      set rc [catch {eval $execute} m]
      
      if {$rc == 0} {
          InsertLog "$m"
      } else {
          InsertLog "PeakToTotal returns an error: $m"
      }
    }
    
   proc do_totalcounts {} {
      variable SpecSize
      variable SpecRawData

       set m [do_read1D [SpectrumName] 0 $SpecSize 1]
       if {[first $m] != 0} {return 0}    ;#   error already reported
       set SpecRawData [second $m]
          
       set total 0
       for {set i 0} {$i < $SpecSize} {incr i} {
          set item [lindex $SpecRawData $i]
          incr total $item
       }
       
       return $total
   }
   
    proc do_maxcounts {} {
      variable SpecSize
      variable SpecRawData
      
       set m [do_read1D [SpectrumName] 0 $SpecSize 1]
       if {[first $m] != 0} {return [list 0 0]}    ;#   error already reported
       set SpecRawData [second $m]
   
       set maximum 0
       set ch 0
       for {set i 0} {$i < $SpecSize} {incr i} {
          set item [lindex $SpecRawData $i]
          if {$item > $maximum} {set maximum $item; set ch $i}
       }
       
       return [list $ch $maximum]
   }
    
   proc do_cont {def} {
       variable DefCont
       if {$def == "true" || $def == 1} {
         set DefCont 1
       } else {
         set DefCont 0
       }
   }

   proc do_skip {skip} {
       variable Skip0
       variable XMin
       variable SpecBase
       if {$skip == "true" || $skip == 1} {
         set Skip0 1
       } else {
         set Skip0 0
         if {$XMin == 1} {set XMin $SpecBase}
       }
   }   
    
    proc do_zoom {w} {
       variable XMin 
       variable XMax
       variable tagx
       variable ntags
    
       if {$ntags < 1} {InsertLog "You need 1 tag for zoom"; return}
    
        set x $tagx($ntags)
        set range [expr $XMax - $XMin]
        
        if {$w == "In"} {
           set XMin [expr $x - ($range/4)]
           set XMax [expr $x + ($range/4)]
        } else {
           set XMin [expr $x - $range]
           set XMax [expr $x + $range]        
        }
    }
   
   proc do_pushtag {x} {
       variable numtag
       variable tagx
       variable tagy
       variable ntags
       variable SpecRawData
       variable Compression
       variable XMin
   
       if {$ntags == $numtag} {
           InsertLog "too many tags. max supported = $numtag"
       } else {
           incr ntags
           set tagx($ntags) [expr round($x)]
           set tagy($ntags) [lindex $SpecRawData [expr ($tagx($ntags)-$XMin)/$Compression]]
       }   
   }
   
   proc do_reset {} {
       variable XMin 
       variable XMax
       variable YMin
       variable YMax
       variable SpecSize
       variable SpecLimit
       variable WindowSize 
   
       set XMin 0
       set XMax $SpecSize
       set WindowSize [expr $XMax - $XMin]
       set YMin 0
       set YMax [expr ($SpecLimit * 11)/10]
   
   }
   
   proc do_expand {} {
       variable XMin 
       variable XMax
       variable tagx
       variable ntags
       variable WindowSize
       
       if {$ntags < 2} {InsertLog "You need 2 tags for expand"; return}
   
       if {$tagx([expr $ntags-1]) < $tagx($ntags)} {
           set XMin $tagx([expr $ntags-1])
           set XMax $tagx($ntags)
       } else {
           set XMin $tagx($ntags)
           set XMax $tagx([expr $ntags-1])
       }
       
       set WindowSize [expr $XMax - $XMin]
       
       do_poptag
       do_poptag
   }
   
   proc do_cleartags {} {
       variable ntags
       variable numtag
       variable tagx
       variable tagy
   
       set ntags 0
       for {set i 1} {$i<=$numtag} {incr i} {set tagx($i) 0; set tagy($i) 0}
   
   }
   
   proc do_poptag {} {
       variable ntags
       variable tagx
       variable tagy
   
       if {$ntags < 1} {return}
       
       set tagx($ntags) 0
       set tagy($ntags) 0
       incr ntags -1       
   }

